#!/usr/bin/env python
from __future__ import print_function
import sys
import os
import struct
try:
    import usocket as socket
except ImportError:
    import socket
import oneiot_core.websocket_helper as websocket_helper

# Define to 1 to use builtin "uwebsocket" module of MicroPython
USE_BUILTIN_UWEBSOCKET = 0
# Treat this remote directory as a root for file transfers
SANDBOX = ""
#SANDBOX = "/tmp/webrepl/"
DEBUG = 0

WEBREPL_REQ_S = "<2sBBQLH64s"
WEBREPL_PUT_FILE = 1
WEBREPL_GET_FILE = 2
WEBREPL_GET_VER  = 3


def debugmsg(msg):
    if DEBUG:
        print(msg)


if USE_BUILTIN_UWEBSOCKET:
    from uwebsocket import websocket
else:
    class websocket:

        def __init__(self, s):
            self.s = s
            self.buf = b""

        def write(self, data):
            l = len(data)
            if l < 126:
                # TODO: hardcoded "binary" type
                hdr = struct.pack(">BB", 0x82, l)
            else:
                hdr = struct.pack(">BBH", 0x82, 126, l)
            self.s.send(hdr)
            self.s.send(data)

        def recvexactly(self, sz):
            res = b""
            while sz:
                data = self.s.recv(sz)
                if not data:
                    break
                res += data
                sz -= len(data)
            return res

        def read(self, size, text_ok=False):
            if not self.buf:
                while True:
                    hdr = self.recvexactly(2)
                    assert len(hdr) == 2
                    fl, sz = struct.unpack(">BB", hdr)
                    if sz == 126:
                        hdr = self.recvexactly(2)
                        assert len(hdr) == 2
                        (sz,) = struct.unpack(">H", hdr)
                    if fl == 0x82:
                        break
                    if text_ok and fl == 0x81:
                        break
                    debugmsg("Got unexpected websocket record of type %x, skipping it" % fl)
                    while sz:
                        skip = self.s.recv(sz)
                        debugmsg("Skip data: %s" % skip)
                        sz -= len(skip)
                data = self.recvexactly(sz)
                assert len(data) == sz
                self.buf = data

            d = self.buf[:size]
            self.buf = self.buf[size:]
            assert len(d) == size, len(d)
            return d

        def ioctl(self, req, val):
            assert req == 9 and val == 2


def login(ws, passwd):
    while True:
        c = ws.read(1, text_ok=True)
        if c == b":":
            assert ws.read(1, text_ok=True) == b" "
            break
    ws.write(passwd.encode("utf-8") + b"\r")

def read_resp(ws):
    data = ws.read(4)
    sig, code = struct.unpack("<2sH", data)
    assert sig == b"WB"
    return code


def send_req(ws, op, sz=0, fname=b""):
    rec = struct.pack(WEBREPL_REQ_S, b"WA", op, 0, 0, sz, len(fname), fname)
    debugmsg("%r %d" % (rec, len(rec)))
    ws.write(rec)


def get_ver(ws):
    send_req(ws, WEBREPL_GET_VER)
    d = ws.read(3)
    d = struct.unpack("<BBB", d)
    return d


def put_file(ws, local_file, remote_file):
    sz = os.stat(local_file)[6]
    dest_fname = (SANDBOX + remote_file).encode("utf-8")
    rec = struct.pack(WEBREPL_REQ_S, b"WA", WEBREPL_PUT_FILE, 0, 0, sz, len(dest_fname), dest_fname)
    debugmsg("%r %d" % (rec, len(rec)))
    ws.write(rec[:10])
    ws.write(rec[10:])
    assert read_resp(ws) == 0
    cnt = 0
    with open(local_file, "rb") as f:
        while True:
            sys.stdout.write("Sent %d of %d bytes\r" % (cnt, sz))
            sys.stdout.flush()
            buf = f.read(1024)
            if not buf:
                break
            ws.write(buf)
            cnt += len(buf)
    print()
    assert read_resp(ws) == 0

def get_file(ws, local_file, remote_file):
    src_fname = (SANDBOX + remote_file).encode("utf-8")
    rec = struct.pack(WEBREPL_REQ_S, b"WA", WEBREPL_GET_FILE, 0, 0, 0, len(src_fname), src_fname)
    debugmsg("%r %d" % (rec, len(rec)))
    ws.write(rec)
    assert read_resp(ws) == 0
    with open(local_file, "wb") as f:
        cnt = 0
        while True:
            ws.write(b"\0")
            (sz,) = struct.unpack("<H", ws.read(2))
            if sz == 0:
                break
            while sz:
                buf = ws.read(sz)
                if not buf:
                    raise OSError()
                cnt += len(buf)
                f.write(buf)
                sz -= len(buf)
                sys.stdout.write("Received %d bytes\r" % cnt)
                sys.stdout.flush()
    print()
    assert read_resp(ws) == 0


def help(rc=0):
    exename = sys.argv[0].rsplit("/", 1)[-1]
    print("%s - Perform remote file operations using MicroPython WebREPL protocol" % exename)
    print("Arguments:")
    print("  [-p password] <host>:<remote_file> <local_file> - Copy remote file to local file")
    print("  [-p password] <local_file> <host>:<remote_file> - Copy local file to remote file")
    print("Examples:")
    print("  %s script.py 192.168.4.1:/another_name.py" % exename)
    print("  %s script.py 192.168.4.1:/app/" % exename)
    print("  %s -p password 192.168.4.1:/app/script.py ." % exename)
    sys.exit(rc)

def error(msg):
    print(msg)
    sys.exit(1)

def parse_remote(remote):
    host, fname = remote.rsplit(":", 1)
    if fname == "":
        fname = "/"
    port = 8266
    if ":" in host:
        host, port = host.split(":")
        port = int(port)
    return (host, port, fname)

#op = 'get'/'put'
def main(passwd, remote, op, dst_file = None, src_file = None):

    if op == "get":
        host, port, src_file = parse_remote(remote)
        if os.path.isdir(dst_file):
            basename = src_file.rsplit("/", 1)[-1]
            dst_file += "/" + basename
    else:
        host, port, dst_file = parse_remote(remote)
        if dst_file[-1] == "/":
            basename = src_file.rsplit("/", 1)[-1]
            dst_file += basename

    if True:
        print("op:%s, host:%s, port:%d, passwd:%s." % (op, host, port, passwd))
        print(src_file, "->", dst_file)

    s = socket.socket()

    ai = socket.getaddrinfo(host, port)
    addr = ai[0][4]

    s.connect(addr)
    websocket_helper.client_handshake(s)

    ws = websocket(s)

    login(ws, passwd)
    print("Remote WebREPL version:", get_ver(ws))

    # Set websocket to send data marked as "binary"
    ws.ioctl(9, 2)

    if op == "get":
        get_file(ws, dst_file, src_file)
    elif op == "put":
        put_file(ws, src_file, dst_file)

    s.close()



from setuptools import setup, find_packages
from setuptools.command.install import install

with open("README.md", "r") as fh:
    long_description = fh.read()

class CustomInstallCommand(install):
    def run(self):
        # import os, re, sys, subprocess, getpass
        # try:
        #     import httplib
        # except:
        #     import http.client as httplib
        # import datetime
        # import configparser
        #
        # def print_message(status, message, include_time = False):
        #     print('[' + status + ']\t' + message + ('\t' + str(datetime.datetime.now().strftime('%M:%S.%f')) if include_time else ''))
        #
        # def check_internet_connection():
        #     print_message('INFO', 'Checking for an active internet connection...')
        #     while True:
        #         conn = httplib.HTTPConnection("www.google.com", timeout=5)
        #         try:
        #             conn.request("HEAD", "/")
        #             conn.close()
        #             print_message('INFO', 'Internet connection found!')
        #             return True
        #         except:
        #             conn.close()
        #             print_message('WARN', 'No internet connection found. Please connect to the internet and press [ENTER] to continue.')
        #             input("")
        #
        # def is_valid_network_name(name):
        #     return re.match('^\w+$', name)
        #
        # if not __name__ == '__main__':
        #     print_message('FATAL', 'Not executing as top level code')
        #     exit()
        #
        # os.system("clear")
        # print("OneIoT Core setup")
        # print()
        # print("Please ensure that you are connected to the internet")
        # print("Press [ENTER] to continue")
        # input()
        #
        # # Check internet connection
        # check_internet_connection()
        #
        # #Hotspot setup
        # print_message("INFO", "Setting up wireless hotspot\n")
        # network_name = input("Enter a name for the wireless network used to connect smart device (default: OneIoT): ")
        # if network_name == "":
        #     network_name = "OneIoT"
        # while not is_valid_network_name(network_name):
        #     network_name = input("Invalid network name, please enter only alphanumeric characters, or press enter to use the default value:")
        #     if network_name == "":
        #         network_name = "OneIoT"
        #
        # network_password = getpass.getpass("Enter a secure password: ")
        # while network_password == "":
        #     network_password = getpass.getpass("Enter a secure password: ")
        #
        # confirm_password = getpass.getpass("Confirm password: ")
        # while confirm_password != network_password:
        #     confirm_password = getpass.getpass("Confirm password: ")
        #
        # #Configure static ip
        # os.system("printf 'interface wlan0\n\tstatic ip_address=192.168.4.1/24\n\tnohook wpa_supplicant' | sudo tee -a /etc/dhcpcd.conf")
        # os.system("sudo systemctl restart dhcpcd")
        #
        # #Configure hostapd
        # hostapd_config = "interface=wlan0\ndriver=nl80211\nssid=" + network_name + "\nhw_mode=g\nchannel=6\nieee80211n=1\nwmm_enabled=1\nht_capab=[HT40][SHORT-GI-20][DSSS_CCK-40]\nmacaddr_acl=0\nauth_algs=1\nignore_broadcast_ssid=0\nwpa=2\nwpa_key_mgmt=WPA-PSK\nwpa_passphrase=" + network_password + "\nrsn_pairwise=CCMP"
        # os.system("printf '" + hostapd_config + "' | sudo tee /etc/hostapd/hostapd.conf")
        # os.system("printf 'DAEMON_CONF=\"/etc/hostapd/hostapd.conf\"' | sudo tee -a /etc/default/hostapd")
        #
        # #Configure dnsmasq
        # dnsmasq_config = "interface=wlan0\nbind-interfaces\nserver=8.8.8.8\ndomain-needed\nbogus-priv\ndhcp-range=192.168.4.2,192.168.4.20,255.255.255.0,24h"
        # os.system("sudo mv /etc/dnsmasq.conf /etc/dnsmasq.conf.orig")
        # os.system("sudo touch /etc/dnsmasq.conf")
        # os.system("printf '" + dnsmasq_config + "' | sudo tee /etc/dnsmasq.conf")
        #
        # #IPV4 forwarding
        # os.system("printf 'net.ipv4.ip_forward=1' | sudo tee -a /etc/sysctl.conf")
        # os.system("sudo sh -c 'echo 1 > /proc/sys/net/ipv4/ip_forward'")
        #
        # #Restart services
        # os.system("sudo systemctl unmask hostapd")
        # os.system("sudo systemctl enable hostapd")
        # os.system("sudo systemctl start hostapd")
        # os.system("sudo service dnsmasq start")
        # print_message("INFO", "Completed setting up wireless hotspot")
        #
        # config = configparser.ConfigParser()
        # config['wireless-info'] = {'ssid':network_name, 'psk':network_password}
        #
        # oneIot_path = os.path.expanduser("~/.oneIot")
        #
        # if not os.path.exists(oneIot_path):
        #     os.makedirs(oneIot_path)
        #
        # with open(oneIot_path + "/config.ini", 'w') as configfile:
        #     config.write(configfile)
        #
        # os.system("clear")
        # print("Device setup complete! please reboot now")

        install.run(self)

setup(
    name="oneiot-core",
    version="0.1.0",
    author="Louis Irwin",
    author_email="coding@louisirwin.co.uk",
    description="OneIoT Core",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lirwin3007/OneIoT-Core",
    packages=find_packages(),
    python_requires='>=3.6',
    cmdclass={
        'install': CustomInstallCommand
    },
    scripts=[
        'scripts/iot-core'
    ],
    entry_points={
        'console_scripts': [
            'iot-core-main = oneiot_core.__main__:main'
        ]
    },
)
